@@ -185,7 +185,7 @@ pub mod stacks {
         ///
         /// ```rust
         /// use stacking::stacks::Stack;
-        /// 
+        ///
         /// let mut stack: Stack<i32> = Stack::new();
         /// stack.push(5);
         /// assert_eq!(stack.len(), 1);
@@ -196,4 +196,111 @@ pub mod stacks {
             self.len
         }
     }
+
+    #[derive(Debug, Clone)]
+    /// a struct representing a queue which is a specific type of stack.
+    /// You push to the top and pop from the bottom.
+    pub struct Queue<T: Clone> {
+        top: Option<Node<T>>,
+        bottom: Option<Node<T>>,
+        len: usize,
+    }
+
+    impl<T: Clone> Queue<T> {
+        /// create a new instance of a Queue
+        /// # Returns
+        /// a new empty instance of a Queue
+        pub fn new() -> Queue<T> {
+            Queue {
+                top: None,
+                bottom: None,
+                len: 0,
+            }
+        }
+
+        /// push a new element to the Queue
+        ///
+        /// # Arguments
+        /// 
+        /// * `val`: the value to push
+        ///
+        /// # Examples
+        ///
+        /// ```rust
+        /// use stacking::stacks::Queue;
+        ///
+        /// let mut queue: Queue<i32> = Queue::new();
+        /// queue.push(4);
+        /// queue.push(5);
+        /// assert_eq!(queue.pop(), Some(4));
+        /// assert_eq!(queue.pop(), Some(5));
+        /// ```
+        pub fn push(&mut self, val: T) {
+            let new_top: Node<T> = Node::new(None, self.top.clone(), val);
+            self.top = Some(new_top.clone());
+            if self.len == 0 {
+                self.bottom = Some(new_top.clone());
+            } else if self.len == 1 {
+                self.bottom = Some(Node::new(self.top.clone(), None, self.bottom.clone().unwrap().get_val()))
+            }
+            self.len += 1;
+        }
+
+        /// pop the element at the bottom and return it's value
+        ///
+        /// # Returns
+        /// the value of the element at the bottom
+        ///
+        /// # Examples
+        ///
+        /// ```rust
+        /// use stacking::stacks::Queue;
+        ///
+        /// let mut queue: Queue<i32> = Queue::new();
+        /// queue.push(4);
+        /// queue.push(5);
+        /// assert_eq!(queue.pop(), Some(4));
+        /// assert_eq!(queue.pop(), Some(5));
+        /// ```
+        pub fn pop(&mut self) -> Option<T> {
+            if self.len == 0 {
+                return None;
+            }
+            match self.bottom.clone() {
+                Some(bottom) => {
+                    let val = bottom.get_val().clone();
+                    let new_bottom = match bottom.clone().get_next() {
+                        Some(mut n) => {
+                            n.set_prev(None);
+                            Some(n)
+                        }
+                        None => None,
+                    };
+                    self.bottom = new_bottom;
+                    self.len -= 1;
+                    Some(val)
+                }
+                None => None,
+            }
+        }
+
+        /// get the length of the current Queue
+        ///
+        /// # Returns
+        /// the length of the current Queue as a `usize`
+        ///
+        /// # Examples
+        ///
+        /// ```rust
+        /// use stacking::stacks::Queue;
+        ///
+        /// let mut queue: Queue<i32> = Queue::new();
+        /// assert_eq!(queue.len(), 0);
+        /// queue.push(4);
+        /// assert_eq!(queue.len(), 1);
+        /// ```
+        pub fn len(&self) -> usize {
+            self.len
+        }
+    }   
 }
